@@ -12,6 +12,7 @@ typedef enum {
   TICKIT_EV_RESIZE = 0x01, // lines, cols
   TICKIT_EV_KEY    = 0x02, // type(TickitKeyEventType), str
   TICKIT_EV_MOUSE  = 0x04, // type(TickitMouseEventType), button, line, col
+  TICKIT_EV_CHANGE = 0x08, // {none}
 
   TICKIT_EV_UNBIND = 0x80000000, // event handler is being unbound
 } TickitEventType;
@@ -48,8 +49,8 @@ typedef struct {
 typedef struct TickitPen TickitPen;
 
 typedef enum {
-  TICKIT_PEN_FG,         /* number: TODO - colour? */
-  TICKIT_PEN_BG,         /* number: TODO - colour? */
+  TICKIT_PEN_FG,         /* colour */
+  TICKIT_PEN_BG,         /* colour */
   TICKIT_PEN_BOLD,       /* bool */
   TICKIT_PEN_UNDER,      /* bool: TODO - number? */
   TICKIT_PEN_ITALIC,     /* bool */
@@ -62,7 +63,8 @@ typedef enum {
 
 typedef enum {
   TICKIT_PENTYPE_BOOL,
-  TICKIT_PENTYPE_INT
+  TICKIT_PENTYPE_INT,
+  TICKIT_PENTYPE_COLOUR,
 } TickitPenAttrType;
 
 TickitPen *tickit_pen_new(void);
@@ -78,11 +80,21 @@ void tickit_pen_set_bool_attr(TickitPen *pen, TickitPenAttr attr, int val);
 int  tickit_pen_get_int_attr(TickitPen *pen, TickitPenAttr attr);
 void tickit_pen_set_int_attr(TickitPen *pen, TickitPenAttr attr, int val);
 
+int  tickit_pen_get_colour_attr(TickitPen *pen, TickitPenAttr attr);
+void tickit_pen_set_colour_attr(TickitPen *pen, TickitPenAttr attr, int value);
+int  tickit_pen_set_colour_attr_desc(TickitPen *pen, TickitPenAttr attr, const char *value);
+
 void tickit_pen_clear_attr(TickitPen *pen, TickitPenAttr attr);
 
 int tickit_pen_equiv_attr(TickitPen *a, TickitPen *b, TickitPenAttr attr);
 
 void tickit_pen_copy_attr(TickitPen *dst, TickitPen *src, TickitPenAttr attr);
+void tickit_pen_copy(TickitPen *dst, TickitPen *src, int overwrite);
+
+typedef void TickitPenEventFn(TickitPen *tt, TickitEventType ev, TickitEvent *args, void *data);
+
+int  tickit_pen_bind_event(TickitPen *tt, TickitEventType ev, TickitPenEventFn *fn, void *data);
+void tickit_pen_unbind_event_id(TickitPen *tt, int id);
 
 TickitPenAttrType tickit_pen_attrtype(TickitPenAttr attr);
 
