@@ -8,7 +8,7 @@ package Tickit;
 use strict;
 use warnings;
 
-our $VERSION = '0.15_001';
+our $VERSION = '0.16_001';
 
 use IO::Handle;
 
@@ -27,18 +27,40 @@ C<Tickit> - Terminal Interface Construction KIT
 =head1 SYNOPSIS
 
  use Tickit;
+ use Tickit::Widget::Box;
+ use Tickit::Widget::Static;
 
  my $tickit = Tickit->new;
 
- # Create some widgets
- # ...
-
- $tickit->set_root_widget( $rootwidget );
+ my $box = Tickit::Widget::Box->new(
+    h_border => 4,
+    v_border => 2,
+    bg       => "green",
+    child    => Tickit::Widget::Static->new(
+       text     => "Hello, world!",
+       bg       => "black",
+       align    => "centre",
+       valign   => "middle",
+    ),
+ );
 
+ $tickit->set_root_widget( $box );
  $tickit->run;
 
 =head1 DESCRIPTION
 
+C<Tickit> is a high-level toolkit for creating full-screen terminal-based
+interactive programs. It allows programs to be written in an abstracted way,
+working with a tree of widget objects, to represent the layout of the
+interface and implement its behaviours.
+
+Its supported terminal features includes a rich set of rendering attributes
+(bold, underline, italic, 256-colours, etc), support for mouse including wheel
+and position events above the 224th column and arbitrary modified key input
+via F<libtermkey> (all of these will require a supporting terminal as well).
+It also supports having multiple instances and non-blocking or asynchronous
+control.
+
 This B<DEVELOPMENT> version of C<Tickit> bundles a snapshot copy of the
 F<libtickit> C library, and builds and links against it. It exists largely as
 an experiment over the (mostly) pure-perl stable release version of C<Tickit>.
