@@ -1,20 +1,23 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
 
 package Tickit::Pen;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.15_001';
+our $VERSION = '0.16_001';
 
 use Carp;
 use Scalar::Util qw( weaken );
 
 our @ALL_ATTRS = qw( fg bg b u i rv strike af );
 
+# Load the XS code
+require Tickit;
+
 =head1 NAME
 
 C<Tickit::Pen> - store a collection of rendering attributes
@@ -92,46 +95,49 @@ sub new_from_attrs
    my $class = shift;
    my ( $attrs ) = @_;
 
-   my $self = bless {
-      _on_changed => [],
-   }, $class;
+   my $self = $class->_new;
 
    $self->chattrs( $attrs );
 
    return $self;
 }
 
+=head2 $pen = $orig->clone
+
+Returns a new pen, initialised by copying the attributes of the original.
+
+=cut
+
+sub clone
+{
+   my $orig = shift;
+   my $new = (ref $orig)->new;
+   $new->copy_from( $orig );
+   return $new;
+}
+
 =head1 METHODS
 
 =cut
 
+=head2 $exists = $pen->hasattr( $attr )
+
+Returns true if the given attribute exists on this object
+
+=cut
+
 =head2 $value = $pen->getattr( $attr )
 
 Returns the current value of the given attribute
 
 =cut
 
-sub getattr
-{
-   my $self = shift;
-   my ( $attr ) = @_;
-
-   return $self->{$attr};
-}
-
 =head2 %values = $pen->getattrs
 
 Returns a key/value list of all the attributes
 
 =cut
 
-sub getattrs
-{
-   my $self = shift;
-
-   return map { $_ => $self->{$_} } grep { !m/^_/ } keys %$self;
-}
-
 =head2 $pen->chattr( $attr, $value )
 
 Change the value of an attribute. Setting C<undef> implies default value. To
@@ -139,70 +145,6 @@ delete an attribute altogether, see instead C<delattr>.
 
 =cut
 
-sub chattr
-{
-   my $self = shift;
-   my ( $attr, $val ) = @_;
-
-   my $canonicalise = $self->can( "_canonicalise_$attr" );
-   $val = $self->$canonicalise( $val ) if $canonicalise;
-
-   # Optimise
-   my $curval = $self->{$attr};
-   return if !defined $curval and !defined $val;
-   return if  defined $curval and  defined $val and $val == $curval;
-
-   $self->{$attr} = $val;
-   $self->_changed;
-}
-
-my @COLOURNAMES = qw(
-   black
-   red
-   green
-   yellow
-   blue
-   magenta
-   cyan
-   white
-);
-
-*_canonicalise_fg = *_canonicalise_bg = \&_canonicalise_colour;
-sub _canonicalise_colour
-{
-   my ( undef, $colour ) = @_;
-
-   return undef if !defined $colour;
-
-   return $colour if $colour =~ m/^\d+$/;
-
-   my $high = ( $colour =~ s/^hi-// ) * 8;
-
-   return $high+$colour if $colour =~ m/^\d+$/ and $colour < 8;
-
-   foreach my $num ( 0 .. $#COLOURNAMES ) {
-      return $high+$num if $colour eq $COLOURNAMES[$num];
-   }
-
-   croak "Unrecognised colour value $colour";
-}
-
-{
-   no strict 'refs';
-   *{"_canonicalise_$_"} = \&_canonicalise_bool for qw( b u i rv strike );
-}
-sub _canonicalise_bool
-{
-   my ( undef, $val ) = @_;
-   return $val ? 1 : undef;
-}
-
-sub _canonicalise_af
-{
-   my ( undef, $val ) = @_;
-   return $val =~ m/^\d$/ ? $val : undef;
-}
-
 =head2 $pen->chattrs( \%attrs )
 
 Change the values of all the attributes given in the hash. Recgonised
@@ -210,14 +152,6 @@ attributes will be deleted from the hash.
 
 =cut
 
-sub chattrs
-{
-   my $self = shift;
-   my ( $attrs ) = @_;
-
-   exists $attrs->{$_} and $self->chattr( $_, delete $attrs->{$_} ) for @ALL_ATTRS;
-}
-
 =head2 $pen->delattr( $attr )
 
 Delete an attribute from this pen. This attribute will no longer be modified
@@ -225,20 +159,32 @@ by this pen.
 
 =cut
 
-sub delattr
+=head2 $pen->copy_from( $other )
+
+=head2 $pen->default_from( $other )
+
+Copy attributes from the given pen. C<copy_from> will override attributes
+already defined by C<$pen>; C<default_from> will only copy attributes that are
+not yet defined by C<$pen>.
+
+As a convenience both methods return C<$pen>.
+
+=cut
+
+sub copy_from
 {
    my $self = shift;
-   my ( $attr ) = @_;
-
-   delete $self->{$attr};
-   $self->_changed;
+   my ( $other ) = @_;
+   $self->copy( $other, 1 );
+   return $self;
 }
 
-sub _changed
+sub default_from
 {
    my $self = shift;
-
-   $_->[0]->on_pen_changed( $self, $_->[1] ) for @{ $self->{_on_changed} };
+   my ( $other ) = @_;
+   $self->copy( $other, 0 );
+   return $self;
 }
 
 =head2 $pen->add_on_changed( $observer, $id )
@@ -256,35 +202,12 @@ not weakened.
 
 =cut
 
-sub add_on_changed
-{
-   my $self = shift;
-   my ( $observer, $id ) = @_;
-
-   push @{ $self->{_on_changed} }, [ $observer, $id ];
-   weaken $self->{_on_changed}[-1][0];
-}
-
 =head2 $pen->remove_on_changed( $observer )
 
 Remove an observer previously added by C<add_on_changed>.
 
 =cut
 
-sub remove_on_changed
-{
-   my $self = shift;
-   my ( $observer ) = @_;
-
-   # Can't grep() because that would strengthen weakrefs
-   my $on_changed = $self->{_on_changed};
-   for( my $i = 0; $i < @$on_changed; ) {
-      # Be well-behaved at global destruction time
-      $i++, next unless !defined $on_changed->[$i][0] or $on_changed->[$i][0] == $observer;
-      splice @$on_changed, $i, 1, ();
-   }
-}
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
