@@ -8,7 +8,7 @@ package Tickit::Term;
 use strict;
 use warnings;
 
-our $VERSION = '0.15_001';
+our $VERSION = '0.16_001';
 
 # Load the XS code
 require Tickit;
@@ -229,6 +229,8 @@ undefined if this method returns successful.
 
 =cut
 
+=head2 $term->chpen( $pen )
+
 =head2 $term->chpen( %attrs )
 
 Changes the current pen attributes to those given. Any attribute whose value
@@ -238,11 +240,7 @@ For details of the supported pen attributes, see L<Tickit::Pen>.
 
 =cut
 
-sub chpen
-{
-   my $self = shift;
-   $self->{ttxs}->chpen( @_ );
-}
+=head2 $term->setpen( $pen )
 
 =head2 $term->setpen( %attrs )
 
