@@ -3,7 +3,7 @@ package Tickit::Test::MockTerm;
 use strict;
 use warnings;
 
-our $VERSION = '0.15_001';
+our $VERSION = '0.16_001';
 
 sub new
 {
@@ -297,7 +297,7 @@ sub _deletech
 sub chpen
 {
    my $self = shift;
-   my %attrs = @_;
+   my %attrs = ( @_ == 1 ) ? shift->getattrs : @_;
 
    $self->{pen}{$_} = $attrs{$_} for keys %attrs;
 
@@ -307,7 +307,7 @@ sub chpen
 sub setpen
 {
    my $self = shift;
-   my %attrs = @_;
+   my %attrs = ( @_ == 1 ) ? shift->getattrs : @_;
    $self->chpen( map { $_ => $attrs{$_} } @Tickit::Pen::ALL_ATTRS );
 }
 
