@@ -1,4 +1,5 @@
 #include "tickit.h"
+#include "hooklists.h"
 
 /* We need C99 vsnprintf() semantics */
 #define _ISOC99_SOURCE
@@ -37,14 +38,6 @@ static inline int terminfo_columns(void) { return columns; }
 
 #include <termkey.h>
 
-struct TickitTermEventHook {
-  struct TickitTermEventHook *next;
-  TickitEventType             ev;
-  TickitTermEventFn          *fn;
-  void                       *data;
-  int                         id;
-};
-
 struct TickitTerm {
   int                   outfd;
   TickitTermOutputFunc *outfunc;
@@ -69,14 +62,17 @@ struct TickitTerm {
 
   TickitPen *pen;
 
-  struct TickitTermEventHook *hooks;
+  struct TickitEventHook *hooks;
 };
 
-static void run_events(TickitTerm *tt, TickitEventType ev, TickitEvent *args)
+DEFINE_HOOKLIST_FUNCS(term,TickitTerm,TickitTermEventFn)
+
+static TermKey *get_termkey(TickitTerm *tt)
 {
-  for(struct TickitTermEventHook *hook = tt->hooks; hook; hook = hook->next)
-    if(hook->ev & ev)
-      (*hook->fn)(tt, ev, args, hook->data);
+  if(!tt->termkey)
+    tt->termkey = termkey_new(tt->infd, TERMKEY_FLAG_EINTR);
+
+  return tt->termkey;
 }
 
 TickitTerm *tickit_term_new(void)
@@ -148,14 +144,7 @@ TickitTerm *tickit_term_new_for_termtype(const char *termtype)
 
 void tickit_term_free(TickitTerm *tt)
 {
-  for(struct TickitTermEventHook *hook = tt->hooks; hook;) {
-    struct TickitTermEventHook *next = hook->next;
-    if(hook->ev & TICKIT_EV_UNBIND)
-      (*hook->fn)(tt, TICKIT_EV_UNBIND, NULL, hook->data);
-    free(hook);
-    hook = next;
-  }
-
+  tickit_hooklist_unbind_and_destroy(tt->hooks, tt);
   tickit_pen_destroy(tt->pen);
 
   if(tt->termkey)
@@ -227,9 +216,11 @@ void tickit_term_set_output_func(TickitTerm *tt, TickitTermOutputFunc *fn, void
 
 void tickit_term_set_input_fd(TickitTerm *tt, int fd)
 {
-  //: TODO: some way to signal an error
-  if(!tt->termkey)
-    tt->infd = fd;
+  if(tt->termkey)
+    termkey_destroy(tt->termkey);
+
+  tt->infd = fd;
+  (void)get_termkey(tt);
 }
 
 int tickit_term_get_input_fd(TickitTerm *tt)
@@ -237,14 +228,6 @@ int tickit_term_get_input_fd(TickitTerm *tt)
   return tt->infd;
 }
 
-static TermKey *get_termkey(TickitTerm *tt)
-{
-  if(!tt->termkey)
-    tt->termkey = termkey_new(tt->infd, TERMKEY_FLAG_EINTR);
-
-  return tt->termkey;
-}
-
 static void got_key(TickitTerm *tt, TermKey *tk, TermKeyKey *key)
 {
   TickitEvent args;
@@ -505,19 +488,18 @@ int tickit_term_scrollrect(TickitTerm *tt, int top, int left, int lines, int col
 
   if(left == 0 && cols == tt->cols && rightward == 0) {
     write_strf(tt, "\e[%d;%dr", top + 1, top + lines);
+    tickit_term_goto(tt, top, left);
     if(downward > 0) {
-      tickit_term_goto(tt, top + lines - 1, -1);
-      if(downward > 4)
-        write_strf(tt, "\e[%dT", downward);
-      else 
-        write_str(tt, "\n\n\n\n", downward);
+      if(downward > 1)
+        write_strf(tt, "\e[%dM", downward); /* DL */
+      else
+        write_str(tt, "\e[M", 3);
     }
     else {
-      tickit_term_goto(tt, top, -1);
-      if(downward < -2)
-        write_strf(tt, "\e[%dS", -downward);
+      if(downward < -1)
+        write_strf(tt, "\e[%dL", -downward); /* IL */
       else
-        write_str(tt, "\eM\eM", 2 * -downward);
+        write_str(tt, "\e[L", 3);
     }
     write_str(tt, "\e[r", 3);
     return 1;
@@ -571,7 +553,7 @@ static void do_pen(TickitTerm *tt, TickitPen *pen, int ignoremissing)
     switch(attr) {
     case TICKIT_PEN_FG:
     case TICKIT_PEN_BG:
-      val = tickit_pen_get_int_attr(pen, attr);
+      val = tickit_pen_get_colour_attr(pen, attr);
       if(val < 0)
         params[pindex++] = onoff->off;
       else if(val < 8)
@@ -627,7 +609,8 @@ static void do_pen(TickitTerm *tt, TickitPen *pen, int ignoremissing)
 
   s += sprintf(s, "\e[");
   for(int i = 0; i < pindex-1; i++)
-    s += sprintf(s, "%d%c", params[i]&0x7fffffff, params[i]&0x80000000 ? ':' : ';');
+    /* TODO: Work out what terminals support :s */
+    s += sprintf(s, "%d%c", params[i]&0x7fffffff, ';');
   if(pindex > 0)
     s += sprintf(s, "%d", params[pindex-1]&0x7fffffff);
   sprintf(s, "m");
@@ -706,41 +689,3 @@ void tickit_term_set_mode_mouse(TickitTerm *tt, int on)
   write_str(tt, on ? "\e[?1002h" : "\e[?1002l", 0);
   tt->mode.mouse = !!on;
 }
-
-int tickit_term_bind_event(TickitTerm *tt, TickitEventType ev, TickitTermEventFn *fn, void *data)
-{
-  int max_id = 0;
-
-  /* Find the end of a linked list, and find the highest ID in use while we're
-   * at it
-   */
-  struct TickitTermEventHook **newhook = &tt->hooks;
-  for(; *newhook; newhook = &(*newhook)->next)
-    if((*newhook)->id > max_id)
-      max_id = (*newhook)->id;
-
-  *newhook = malloc(sizeof(struct TickitTermEventHook)); // TODO: malloc failure
-
-  (*newhook)->next = NULL;
-  (*newhook)->ev = ev;
-  (*newhook)->fn = fn;
-  (*newhook)->data = data;
-
-  return (*newhook)->id = max_id + 1;
-}
-
-void tickit_term_unbind_event_id(TickitTerm *tt, int id)
-{
-  struct TickitTermEventHook **link = &tt->hooks;
-  for(struct TickitTermEventHook *hook = tt->hooks; hook;) {
-    if(hook->id == id) {
-      *link = hook->next;
-      if(hook->ev & TICKIT_EV_UNBIND)
-        (*hook->fn)(tt, TICKIT_EV_UNBIND, NULL, hook->data);
-      free(hook);
-      hook = *link;
-    }
-    else
-      hook = hook->next;
-  }
-}
